# Copyright 2016, 2017 Lingfei Wang
# 
# This file is part of Findr.
# 
# Findr is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Findr is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with Findr.  If not, see <http://www.gnu.org/licenses/>.
# 
"""Examples of findr. For geuvadis dataset, see findr.examples.geuvadis1, findr.examples.geuvadis2, ... , findr.examples.geuvadis4"""

def load_geuvadis_data():
	"""This function loads downsampled data files from the Geuvadis study (Lappalainen, T. et al. Transcriptome and genome sequencing uncovers functional variation in humans. Nature 501, 506-511 (2013)), including expression levels of 10 miRNAs and 3000 genes for 360 European individuals. Among them, all miRNAs and 1000 genes have significant cis-eQTLs, whose haplotypes are also included. File data formats follow Findr's binary interface input/output requirement. A description of each file is available below:
	dmi.dat:	Expression levels of 10 miRNAs
	dgmi.dat:	Haplotypes of cis-eQTLs of 10 miRNAs
	dc.dat:		Continuous causal anchors for demonstration purposes, simulated from adding continuous noise to dgmi.dat
	dt.dat:		Expression levels of 1000 genes that have cis-eQTLs
	dt2.dat:	Expression levels of 3000 genes
	dgt.dat:	Haplotypes of cis-eQTLs of 1000 genes
	namest.txt:	3000 gene names"""
	from os.path import dirname,join
	from .auto import gtype_np,ftype_np
	import numpy as np
	def getdata(name,dtype,shape):
		d=join(dirname(__file__),'data','geuvadis',name)
		d=np.fromfile(d,dtype=dtype)
		d=d.reshape(*shape)
		return d
	
	ans={'dc':getdata('dc.dat',ftype_np,(10,360)),
	'dgmi':getdata('dgmi.dat',gtype_np,(10,360)),
	'dmi':getdata('dmi.dat',ftype_np,(10,360)),
	'dgt':getdata('dgt.dat',gtype_np,(1000,360)),
	'dt':getdata('dt.dat',ftype_np,(1000,360)),
	'dt2':getdata('dt2.dat',ftype_np,(3000,360))}
	
	f=open(join(dirname(__file__),'data','geuvadis','namest.txt'),'r')
	namest=map(lambda x:x.strip('\r\n'),f.readlines())
	f.close()
	ans['namest']=namest
	return ans
	
def geuvadis1():
	"""Perform the correlation test, without genotype data, from 10 miRNAs to 1000 genes"""
	lines=['import findr,findr.examples',
		'l=findr.lib() # or verbose: l=findr.lib(loglv=12)',
		'd=findr.examples.load_geuvadis_data()',
		'ans=l.pij_rank(d["dmi"],d["dt"])']
	for line in lines:
		print '# '+line
		exec line
	print '# return ans'
	return ans
	
def geuvadis2():
	"""Perform the novel causal inference test from 10 miRNAs to 1000 genes, using cis-eQTLs as causal anchors"""
	lines=['import findr,findr.examples',
		'l=findr.lib()',
		'd=findr.examples.load_geuvadis_data()',
		'ans=l.pij_gassist(d["dgmi"],d["dmi"],d["dt"])']
	for line in lines:
		print '# '+line
		exec line
	print '# return ans'
	return ans

def geuvadis3():
	"""Perform the novel causal inference test from 1000 genes with cis-eQTLs to all 3000 genes"""
	lines=['import findr,findr.examples',
		'l=findr.lib()',
		'd=findr.examples.load_geuvadis_data()',
		'ans=l.pij_gassist(d["dgt"],d["dt"],d["dt2"],nodiag=True)']
	for line in lines:
		print '# '+line
		exec line
	print '# return ans'
	return ans

def geuvadis4():
	"""Perform 5 subtests for causal inference test from 10 miRNAs to 1000 genes, using cis-eQTLs as causal anchors"""
	lines=['import findr,findr.examples',
		'l=findr.lib()',
		'd=findr.examples.load_geuvadis_data()',
		'ans=l.pijs_gassist(d["dgmi"],d["dmi"],d["dt"])']
	for line in lines:
		print '# '+line
		exec line
	print '# return ans'
	return ans

def geuvadis5():
	"""Perform the novel causal inference test from 10 miRNAs to 1000 genes, using continuous causal anchors"""
	lines=['import findr,findr.examples',
		'l=findr.lib()',
		'd=findr.examples.load_geuvadis_data()',
		'ans=l.pij_cassist(d["dc"],d["dmi"],d["dt"])']
	for line in lines:
		print '# '+line
		exec line
	print '# return ans'
	return ans

def geuvadis6():
	"""Perform 5 subtests for causal inference test from 10 miRNAs to 1000 genes, using cis-eQTLs as causal anchors and only obtaining p-values"""
	lines=['import findr,findr.examples',
		'l=findr.lib()',
		'd=findr.examples.load_geuvadis_data()',
		'ans=l.pijs_gassist_pv(d["dgmi"],d["dmi"],d["dt"])']
	for line in lines:
		print '# '+line
		exec line
	print '# return ans'
	return ans

def geuvadis7():
	"""Constructs maximal direct acyclic graph from pairwise causal inference test probability among 1000 genes with cis-eQTLs"""
	lines=['import findr,findr.examples',
		'l=findr.lib()',
		'd=findr.examples.load_geuvadis_data()',
		'ans1=l.pij_gassist(d["dgt"],d["dt"],d["dt"],nodiag=True)',
		'ans2=l.netr_one_greedy(ans1["p"])']
	for line in lines:
		print '# '+line
		exec line
	print '# return ans2'
	return ans2
























# Copyright 2016, 2017 Lingfei Wang
# 
# This file is part of Findr.
# 
# Findr is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Findr is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with Findr.  If not, see <http://www.gnu.org/licenses/>.
# 
"""Python interface"""

def one_greedy(self,dp,namax=None,nimax=None,nomax=None):
	"""Reconstructs a directed acyclic graph according to prior information of edge significance.
	This function first ranks all edges and introduce the most significant one by one, avoiding
	those that would create a loop. Optional constraints on the maximum total number of edges,
	the number of incoming or outgoing edges for every gene can be specified.
	dp:	numpy.ndarray(nt,nt,dtype=ftype(='f4' by default))
		Prior information of edge significance levels. Entry dp[i,j] is significance of edge i to j. 
		A larger values indicates the edge's presence is more probable.
		One option to obtain the prior information is to use pairwise inference methods in findr.
	dt2:numpy.ndarray(nt2,ns,dtype=ftype(='=f4' by default)) Gene expression data for B.
		dt2 has the same format as dt, and can be identical with, different from, or a superset of dt.
		When dt2 is a superset of (or identical with) dt, dt2 must be arranged
		to be identical with dt at its upper submatrix, i.e. dt2[:nt,:]=dt, and
		set parameter nodiag = 1.
	namax:	Constraint on the maximum total number of edges in the reconstructed network.
	nimax:	Constraint on the maximum number of incoming edges for each node in the reconstructed
		network.
	nomax:	Constraint on the maximum number of outgoing edges for each node in the reconstructed
		network.
	Return:	dictionary with following keys:
	ret:0 iff execution succeeded.
	net:	numpy.ndarray((nt,nt),dtype=bool). The reconstructed direct acyclic graph or network
		net[i,j]=True if an edge from i to j exists in the reconstructed network, and False otherwise.
	ftype and gtype can be found in auto.py.
	
	Example: see findr.examples.geuvadis7
	"""
	from exceptions import ValueError
	if self.lib is None:
		raise ValueError("Not initialized.")
	import numpy as np
	from .auto import ftype_np
	if dp.dtype.char!=ftype_np:
		raise ValueError('Wrong input dtype for prior matrix')
	if len(dp.shape)!=2:
		raise ValueError('Wrong input shape')
	if not (namax is None or type(namax) is int or type(namax) is long):
		raise ValueError('Wrong namax type')
	if namax is not None and namax<=0:
		raise ValueError('Input requires namax>0.')
	if namax is None:
		namax=-1
	if not (nimax is None or type(nimax) is int or type(nimax) is long):
		raise ValueError('Wrong nimax type')
	if nimax is not None and nimax<=0:
		raise ValueError('Input requires nimax>0.')
	if nimax is None:
		nimax=-1
	if not (nomax is None or type(nomax) is int or type(nomax) is long):
		raise ValueError('Wrong nomax type')
	if nomax is not None and nomax<=0:
		raise ValueError('Input requires nomax>0.')
	if nomax is None:
		nomax=-1
	
	nt=dp.shape[0]
	if nt==0:
		raise ValueError('Invalid prior dimension')
	if dp.shape[1]!=nt:
		raise ValueError('Wrong input shape')
	if np.isnan(dp).sum()>0:
		raise ValueError('NaN found.')
	func=self.cfunc('netr_one_greedy',rettype='size_t',argtypes=['const MATRIXF*','MATRIXUC*','size_t','size_t','size_t'])
	d=np.require(np.zeros((nt,nt),dtype='u1'),requirements=['A','C','O','W'])
	dpr=np.require(dp,requirements=['A','C','O','W'])
	ret=func(dpr,d,namax,nimax,nomax)
	d=d.astype(bool)
	ret=(ret==0)
	ans={'ret':ret,'net':d}
	return ans

































# Copyright 2016, 2017 Lingfei Wang
# 
# This file is part of Findr.
# 
# Findr is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Findr is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with Findr.  If not, see <http://www.gnu.org/licenses/>.
# 
"""Python interface.
For interface class, see: findr.lib.
For examples, see: findr.examples.
"""

import ctypes
__all__=["auto","common","pij","netr","osdepend","types"]
from . import pij, netr

class lib:
	@staticmethod	
	def default_libpaths():
		from .common import lpaths,libfname
		from os.path import join as pjoin
		from os import environ
		if 'WINDIR' in environ:
			lp=[pjoin(environ['WINDIR'],'System32')]+lpaths
		else:
			lp=lpaths
		return map(lambda x:pjoin(x,libfname),lp)
	def __init__(self,path=None,loglv=6,rs=0,nth=0):
		"""Links and initializes shared library.
		path:	Extra exact file location for shared library
		loglv:	Level of logging output. 1-3: Errors, 4-6: Warnings, 7-9: Infos, 10-12: Debug, 0: Default(6).
		rs:		Initial random seed. Default (0) indicates to use current time.
		nth:	Maximum number of parallel threads. Default (0) indicates to use automatically determined number of cores (not always correct).
		"""
		self.lib=None
		from exceptions import ValueError, OSError
		import logging
		from .auto import pkgname,version
		from .osdepend import fdll,typesizet
		if type(rs) is not int or type(loglv) is not int or type(nth) is not int:
			raise ValueError('Wrong input type')
		if loglv<0 or loglv>12:
			raise ValueError('Wrong log level')
		if nth<0:
			raise ValueError('Wrong number of threads')
		paths=self.default_libpaths()
		if path:
			paths=[path]+paths
		for p in paths:
			try:
				lib=fdll(p)
				ln=ctypes.CFUNCTYPE(ctypes.c_char_p)(('lib_name',lib))()
				lv=ctypes.CFUNCTYPE(ctypes.c_char_p)(('lib_version',lib))()
				lv1=ctypes.CFUNCTYPE(typesizet)(('lib_version1',lib))()
				lv2=ctypes.CFUNCTYPE(typesizet)(('lib_version2',lib))()
				lv3=ctypes.CFUNCTYPE(typesizet)(('lib_version3',lib))()
				pv='.'.join(map(str,version))
				if((ln!=pkgname) or (lv1!=version[0]) or (lv2!=version[1])):
					logging.warning('Located library '+ln+' '+lv+' different from python interface for '+pkgname+' '+pv+' at '+p+'. Skipped.')
					lib=None
					continue
				elif lv3!=version[2]:
					logging.warning('Located library '+ln+' '+lv+' different from python interface for '+pkgname+' '+pv+' at '+p+', but only at patch version. Loaded.')
				lib.lib_init(ctypes.c_ubyte(loglv),ctypes.c_ulong(rs),typesizet(nth))
			except:
				lib=None
				continue
			break
		if lib is None:
			raise OSError("Library not found at default path. Please install/update "+pkgname+' library and python interface, or set library path manually.')
		self.lib=lib
	def cfunc(self,*a,**ka):
		from exceptions import ValueError
		if self.lib is None:
			raise ValueError("Not initialized.")
		from .types import cfunc
		return cfunc(self.lib,*a,**ka)
	pij_gassist=pij.gassist
	pij_gassist_trad=pij.gassist_trad
	pijs_gassist=pij.gassists
	pijs_gassist_pv=pij.gassists_pv
	pij_cassist=pij.cassist
	pij_cassist_trad=pij.cassist_trad
	pijs_cassist=pij.cassists
	pijs_cassist_pv=pij.cassists_pv
	pij_rank=pij.rank
	pij_rank_pv=pij.rank_pv
	netr_one_greedy=netr.one_greedy
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	




# Copyright 2016, 2017 Lingfei Wang
# 
# This file is part of Findr.
# 
# Findr is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Findr is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with Findr.  If not, see <http://www.gnu.org/licenses/>.
# 
"""Python interface for this library."""

import ctypes as c
import numpy as np
from .common import *
from .osdepend import typesizet,npulong
from exceptions import ValueError

class blockuc(c.Structure):
	_fields_=[("size",typesizet),("data",c_ubyte_p)]

class vectoruc(c.Structure):
	_fields_=[("size",typesizet),("stride",typesizet),("data",c_ubyte_p),("block",c.POINTER(blockuc)),("owner",c.c_int)]

class matrixuc(c.Structure):
	_fields_=[("size1",typesizet),("size2",typesizet),("tda",typesizet),("data",c_ubyte_p),("block",c.POINTER(blockuc)),("owner",c.c_int)]

class blockf(c.Structure):
	_fields_=[("size",typesizet),("data",ftype_p)]

class vectorf(c.Structure):
	_fields_=[("size",typesizet),("stride",typesizet),("data",ftype_p),("block",c.POINTER(blockf)),("owner",c.c_int)]

class matrixf(c.Structure):
	_fields_=[("size1",typesizet),("size2",typesizet),("tda",typesizet),("data",ftype_p),("block",c.POINTER(blockf)),("owner",c.c_int)]

class blockg(c.Structure):
	_fields_=[("size",typesizet),("data",gtype_p)]

class vectorg(c.Structure):
	_fields_=[("size",typesizet),("stride",typesizet),("data",gtype_p),("block",c.POINTER(blockg)),("owner",c.c_int)]

class matrixg(c.Structure):
	_fields_=[("size1",typesizet),("size2",typesizet),("tda",typesizet),("data",gtype_p),("block",c.POINTER(blockg)),("owner",c.c_int)]

def vectoruc_pin(d0,req=['A','C','W']):
	if len(d0.shape)!=1:
		raise ValueError('Wrong input shape')
	if d0.dtype.char!='B':
		raise ValueError('Wrong input dtype')
	d=np.require(d0,requirements=req)
	nb=d.dtype.type().nbytes
	sb=blockuc(typesizet(d.ctypes.strides[0]*d.shape[0]/nb),d.ctypes.data_as(c_ubyte_p))
	sv=vectoruc(d.ctypes.shape[0],
		d.ctypes.strides[0]/d.dtype.type().nbytes,
		d.ctypes.data_as(c_ubyte_p),
		c.pointer(sb),
		0)
	return sv
def vectoruc_pout(d):
	from exceptions import NotImplementedError
	raise NotImplementedError

def matrixuc_pin(d0,req=['A','C','W']):
	import numpy as np
	if len(d0.shape)!=2:
		raise ValueError('Wrong input shape')
	if d0.dtype.char!='B':
		raise ValueError('Wrong input dtype')
	d=np.require(d0,requirements=req)
	nb=d.dtype.type().nbytes
	sb=blockuc(typesizet(d.ctypes.strides[0]*d.shape[0]/nb),d.ctypes.data_as(c_ubyte_p))
	sv=matrixuc(d.ctypes.shape[0],
		d.ctypes.shape[1],
		d.ctypes.strides[0]/nb,
		d.ctypes.data_as(c_ubyte_p),
		c.pointer(sb),
		0)
	return sv
def matrixuc_pout(d):
	from exceptions import NotImplementedError
	raise NotImplementedError
	

def vectorf_pin(d0,req=['A','C','W']):
	if len(d0.shape)!=1:
		raise ValueError('Wrong input shape')
	if d0.dtype.char!=ftype_np:
		raise ValueError('Wrong input dtype')
	d=np.require(d0,requirements=req)
	nb=d.dtype.type().nbytes
	sb=blockf(typesizet(d.ctypes.strides[0]*d.shape[0]/nb),d.ctypes.data_as(ftype_p))
	sv=vectorf(d.ctypes.shape[0],
		d.ctypes.strides[0]/d.dtype.type().nbytes,
		d.ctypes.data_as(ftype_p),
		c.pointer(sb),
		0)
	return sv
def vectorf_pout(d):
	from exceptions import NotImplementedError
	raise NotImplementedError

def matrixf_pin(d0,req=['A','C','W']):
	import numpy as np
	if len(d0.shape)!=2:
		raise ValueError('Wrong input shape')
	if d0.dtype.char!=ftype_np:
		raise ValueError('Wrong input dtype')
	d=np.require(d0,requirements=req)
	nb=d.dtype.type().nbytes
	sb=blockf(typesizet(d.ctypes.strides[0]*d.shape[0]/nb),d.ctypes.data_as(ftype_p))
	sv=matrixf(d.ctypes.shape[0],
		d.ctypes.shape[1],
		d.ctypes.strides[0]/nb,
		d.ctypes.data_as(ftype_p),
		c.pointer(sb),
		0)
	return sv
def matrixf_pout(d):
	from exceptions import NotImplementedError
	raise NotImplementedError
	
def vectorg_pin(d0,req=['A','C','W']):
	if len(d0.shape)!=1:
		raise ValueError('Wrong input shape')
	if d0.dtype.char!=gtype_np:
		raise ValueError('Wrong input dtype')
	d=np.require(d0,requirements=req)
	nb=d.dtype.type().nbytes
	sb=blockg(typesizet(d.ctypes.strides[0]*d.shape[0]/nb),d.ctypes.data_as(gtype_p))
	sv=vectorg(d.ctypes.shape,
		d.ctypes.strides[0]/d.dtype.type().nbytes,
		d.ctypes.data_as(gtype_p),
		c.pointer(sb),
		0)
	return sv
def vectorg_pout(d):
	from exceptions import NotImplementedError
	raise NotImplementedError

def matrixg_pin(d0,req=['A','C','W']):
	import numpy as np
	if len(d0.shape)!=2:
		raise ValueError('Wrong input shape')
	if d0.dtype.char!=gtype_np:
		raise ValueError('Wrong input dtype')
	d=np.require(d0,requirements=req)
	nb=d.dtype.type().nbytes
	sb=blockg(typesizet(d.ctypes.strides[0]*d.shape[0]/nb),d.ctypes.data_as(gtype_p))
	sv=matrixg()
	sv=matrixg(d.ctypes.shape[0],
		d.ctypes.shape[1],
		d.ctypes.strides[0]/nb,
		d.ctypes.data_as(gtype_p),
		c.pointer(sb),
		0)
	return sv
def matrixg_pout(d):
	from exceptions import NotImplementedError
	raise NotImplementedError
	

class varpipe:
	"""Variable pipe to process input and return variables for C functions"""
	def __init__(self,fin,fout):
		self.fin=fin
		self.fout=fout
	def pin(self,v):
		"""Process function input variables from python to C format"""
		return self.fin(v)
	def pout(self,v):
		"""Process function output variables from C to python format"""
		return self.fout(v)

vp_byte=varpipe(c.c_byte,lambda x:x)
vp_int=varpipe(c.c_int,lambda x:x)
vp_ulong=varpipe(c.c_ulong,lambda x:x)
vp_ulonglong=varpipe(typesizet,lambda x:x)
vp_sizet=varpipe(typesizet,lambda x:x)
vp_void=varpipe(lambda x:None,lambda x:None)
vp_char_p=varpipe(c.c_char_p,lambda x:x.value)
vp_vectorf=varpipe(vectorf_pin,vectorf_pout)
vp_matrixf=varpipe(matrixf_pin,matrixf_pout)
vp_vectorg=varpipe(vectorg_pin,vectorg_pout)
vp_matrixg=varpipe(matrixg_pin,matrixg_pout)
vp_vectoruc=varpipe(vectoruc_pin,vectoruc_pout)
vp_matrixuc=varpipe(matrixuc_pin,matrixuc_pout)
vp_vectorcf=varpipe(lambda x:vectorf_pin(x,req=['A','C']),vectorf_pout)
vp_matrixcf=varpipe(lambda x:matrixf_pin(x,req=['A','C']),matrixf_pout)
vp_vectorcg=varpipe(lambda x:vectorg_pin(x,req=['A','C']),vectorg_pout)
vp_matrixcg=varpipe(lambda x:matrixg_pin(x,req=['A','C']),matrixg_pout)

vmap={
	'byte':('i1',c.c_byte,vp_byte),
	'int':('i8',c.c_int,vp_int),
	'unsigned long':(npulong,c.c_ulong,vp_ulong),
	'size_t':('u8',typesizet,vp_sizet),
	'void':('u0',None,vp_void),
	'char*':(None,c.c_char_p,vp_char_p),
	'VECTORF*':(ftype_np,c.POINTER(vectorf),vp_vectorf),
	'MATRIXF*':(ftype_np,c.POINTER(matrixf),vp_matrixf),
	'VECTORUC*':('B',c.POINTER(vectoruc),vp_vectoruc),
	'MATRIXUC*':('B',c.POINTER(matrixuc),vp_matrixuc),
	'VECTORG*':(gtype_np,c.POINTER(vectorg),vp_vectorg),
	'MATRIXG*':(gtype_np,c.POINTER(matrixg),vp_matrixg),
	'const VECTORF*':(ftype_np,c.POINTER(vectorf),vp_vectorcf),
	'const MATRIXF*':(ftype_np,c.POINTER(matrixf),vp_matrixcf),
	'const VECTORG*':(gtype_np,c.POINTER(vectorg),vp_vectorcg),
	'const MATRIXG*':(gtype_np,c.POINTER(matrixg),vp_matrixcg)
	}
		
class cfunc:
	def __init__(self,lib,funcname,rettype='void',argtypes=[]):
		self.rt=rettype
		self.at=argtypes
		self.f=c.CFUNCTYPE(vmap[rettype][1],*map(lambda x:vmap[x][1],argtypes))((funcname,lib))
	def __call__(self,*arg):
		a=map(lambda x:vmap[self.at[x]][2].pin(arg[x]),xrange(len(arg)))
		return vmap[self.rt][2].pout(self.f(*a))










































# Copyright 2016, 2017 Lingfei Wang
# 
# This file is part of Findr.
# 
# Findr is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Findr is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with Findr.  If not, see <http://www.gnu.org/licenses/>.
# 

from platform import system
import ctypes
if system()=='Darwin' or system()=='Linux':
	fdll=ctypes.CDLL
	typesizet=ctypes.c_ulong
	npulong='=u8'
elif system()=='Windows':
	fdll=ctypes.WinDLL
	typesizet=ctypes.c_ulonglong
	npulong='=u4'
else:
	raise OSError("Unsupported operating system.")



# Copyright 2016, 2017 Lingfei Wang
# 
# This file is part of Findr.
# 
# Findr is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Findr is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with Findr.  If not, see <http://www.gnu.org/licenses/>.
# 
"""Python interface for this library."""

from .auto import *
import ctypes as c
ftype=getattr(c,ftype_c)
gtype=getattr(c,gtype_c)

ftype_p=c.POINTER(ftype)
gtype_p=c.POINTER(gtype)
c_ubyte_p=c.POINTER(c.c_ubyte)



pkgname="findr"
pkgnamefull="Fast Inference of Networks from Directed Regulations"
version=[1,0,3]
license="AGPL-3"
url="https://github.com/lingfeiwang/findr-python"
urllib="https://github.com/lingfeiwang/findr"
urldoc="https://github.com/lingfeiwang/findr/blob/master/doc.pdf"
author="Lingfei Wang"
author_email="Lingfei.Wang.github@outlook.com"
ftype_np="f"
ftype_c="c_float"
gtype_np="B"
gtype_c="c_ubyte"



def pkg_setup():
	from setuptools import setup
	setup(name=pkgname,
		version='.'.join(map(str,version)),
		author=author,
		author_email=author_email,
		description=pkgnamefull,
		long_description="This is a python interface of "+pkgname+" library. Binary library of "+pkgname+" is required and can be found at "+urllib+". "+pkgname+' ('+pkgnamefull+') is a library to infer gene pairwise regulatory relations and to reconstruct gene regulation networks. A full documentation is available at '+urldoc+'.',
		url=url,
		download_url=url,
		include_package_data=True,
		install_requires=['numpy'],
		classifiers=['Development Status :: 5 - Production/Stable',
			'License :: OSI Approved :: GNU Affero General Public License v3',
			'Environment :: Console',
			'Intended Audience :: End Users/Desktop',
			'Intended Audience :: Science/Research',
			'Operating System :: OS Independent',
			'Programming Language :: Python :: 2.7',
			'Topic :: Scientific/Engineering :: Bio-Informatics',
			'Topic :: Software Development :: User Interfaces'],
		license=license,
		packages=[pkgname],
	)

pkg_setup()



# Copyright 2016, 2017 Lingfei Wang
# 
# This file is part of Findr.
# 
# Findr is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Findr is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with Findr.  If not, see <http://www.gnu.org/licenses/>.
# 
"""Python interface"""

def gassists_pv(self,dg,dt,dt2,na=None,memlimit=-1):
	"""Calculates p-values of gene i regulating gene j with genotype data assisted method with multiple tests.
	dg:	numpy.ndarray(nt,ns,dtype=gtype(='u1' by default)) Genotype data.
		Entry dg[i,j] is genotype i's value for sample j.
		Each value must be among 0,1,...,na.
		Genotype i must be best (and significant) eQTL of gene i (in dt).
	dt:	numpy.ndarray(nt,ns,dtype=ftype(='=f4' by default)) Gene expression data for A
		Entry dt[i,j] is gene i's expression level for sample j.
		Genotype i (in dg) must be best (and significant) eQTL of gene i.
	dt2:numpy.ndarray(nt2,ns,dtype=ftype(='=f4' by default)) Gene expression data for B.
		dt2 has the same format as dt, and can be identical with, different from, or a superset of dt.
	na:	Number of alleles the species have. It determintes the maximum number of values each genotype can take. When unspecified, it is automatically
		determined as the maximum of dg.
	memlimit:	The approximate memory usage limit in bytes for the library.  For datasets require a larger memory, calculation will be split into smaller chunks. If the memory limit is smaller than minimum required, calculation can fail with an error message. memlimit=0 defaults to unlimited memory usage.
	Return:	dictionary with following keys:
	ret:0 iff execution succeeded.
	p1:	numpy.ndarray(nt,dtype=ftype(='f4' by default)). P-values for LLR of test 1.
		Test 1 calculates E(A)->A v.s. E(A)  A.
	p2:	numpy.ndarray((nt,nt2),dtype=ftype(='f4' by default)). P-values for LLR of test 2.
		Test 2 calculates E(A)->A--B with E(A)->B v.s. E(A)->A<-B.
	p3:	numpy.ndarray((nt,nt2),dtype=ftype(='f4' by default)). P-values for LLR of test 3.
		Test 3 calculates E(A)->A--B with E(A)->B v.s. E(A)->A->B.
	p4:	numpy.ndarray((nt,nt2),dtype=ftype(='f4' by default)). P-values for LLR of test 4.
		Test 4 calculates E(A)->A--B with E(A)->B v.s. E(A)->A  B.
	p5:	numpy.ndarray((nt,nt2),dtype=ftype(='f4' by default)). P-values for LLR of test 5.
		Test 5 calculates E(A)->A--B with E(A)->B v.s. B<-E(A)->A.
	For more information on tests, see paper.
	ftype and gtype can be found in auto.py.
	
	Example: see findr.examples.geuvadis6
	"""
	from exceptions import ValueError
	if self.lib is None:
		raise ValueError("Not initialized.")
	import numpy as np
	from .auto import ftype_np,gtype_np
	if dg.dtype.char!=gtype_np:
		raise ValueError('Wrong input dtype for genotype data: dg.dtype.char is '+dg.dtype.char+'!='+gtype_np)
	if dt.dtype.char!=ftype_np or dt2.dtype.char!=ftype_np:
		raise ValueError('Wrong input dtype for gene expression data')
	if len(dg.shape)!=2 or len(dt.shape)!=2 or len(dt2.shape)!=2:
		raise ValueError('Wrong input shape')
	if not (type(memlimit) is int or type(memlimit) is long):
		raise ValueError('Wrong memlimit type')
	if not (na is None or type(na) is int or type(na) is long):
		raise ValueError('Wrong na type')
	if na is not None and na<=0:
		raise ValueError('Input requires na>0.')
	ng=dg.shape[0]
	nt=dt2.shape[0]
	ns=dg.shape[1]
	nvx=na+1 if na else dg.max()+1
	
	if nvx<2:
		raise ValueError('Invalid genotype values')
	if dt.shape!=dg.shape or dt2.shape[1]!=ns:
		raise ValueError('Wrong input shape')
	if np.isnan(dt).sum()+np.isnan(dt2).sum()>0:
		raise ValueError('NaN found.')
	
	arglist=['const MATRIXG*','const MATRIXF*','const MATRIXF*','VECTORF*','MATRIXF*','MATRIXF*','MATRIXF*','MATRIXF*','size_t','size_t']
	dgr=np.require(dg,requirements=['A','C','O','W'])
	dtr=np.require(dt,requirements=['A','C','O','W'])
	dt2r=np.require(dt2,requirements=['A','C','O','W'])
	d1=np.require(np.zeros(ng,dtype=dt.dtype),requirements=['A','C','O','W'])
	d2=np.require(np.zeros((ng,nt),dtype=dt.dtype),requirements=['A','C','O','W'])
	d3=np.require(np.zeros((ng,nt),dtype=dt.dtype),requirements=['A','C','O','W'])
	d4=np.require(np.zeros((ng,nt),dtype=dt.dtype),requirements=['A','C','O','W'])
	d5=np.require(np.zeros((ng,nt),dtype=dt.dtype),requirements=['A','C','O','W'])
	args=[dgr,dtr,dt2r,d1,d2,d3,d4,d5,nvx,memlimit]
	func=self.cfunc('pijs_gassist_pv',rettype='int',argtypes=arglist)
	ret=func(*args)
	ans={'ret':ret,'p1':d1,'p2':d2,'p3':d3,'p4':d4,'p5':d5}
	return ans

def gassists(self,dg,dt,dt2,na=None,nodiag=False,memlimit=-1):
	"""Calculates probability of gene i regulating gene j with genotype data assisted method,
	with multiple tests, by converting log likelihoods into probabilities per A for all B.
	Probabilities are converted from likelihood ratios separately for each A. This gives better
	predictions when the number of secondary targets (dt2) is large. (Check program warnings.)
	dg:	numpy.ndarray(nt,ns,dtype=gtype(='u1' by default)) Genotype data.
		Entry dg[i,j] is genotype i's value for sample j.
		Each value must be among 0,1,...,na.
		Genotype i must be best (and significant) eQTL of gene i (in dt).
	dt:	numpy.ndarray(nt,ns,dtype=ftype(='=f4' by default)) Gene expression data for A
		Entry dt[i,j] is gene i's expression level for sample j.
		Genotype i (in dg) must be best (and significant) eQTL of gene i.
	dt2:numpy.ndarray(nt2,ns,dtype=ftype(='=f4' by default)) Gene expression data for B.
		dt2 has the same format as dt, and can be identical with, different from, or a superset of dt.
		When dt2 is a superset of (or identical with) dt, dt2 must be arranged
		to be identical with dt at its upper submatrix, i.e. dt2[:nt,:]=dt, and
		set parameter nodiag = 1.
	na:	Number of alleles the species have. It determintes the maximum number of values each genotype can take. When unspecified, it is automatically
		determined as the maximum of dg.
	nodiag:	skip diagonal regulations, i.e. regulation A->B for A=B.
		This should be set to True when A is a subset of B and aligned correspondingly.
	memlimit:	The approximate memory usage limit in bytes for the library.  For datasets require a larger memory, calculation will be split into smaller chunks. If the memory limit is smaller than minimum required, calculation can fail with an error message. memlimit=0 defaults to unlimited memory usage.
	Return:	dictionary with following keys:
	ret:0 iff execution succeeded.
	p1:	numpy.ndarray(nt,dtype=ftype(='=f4' by default)). Probability for test 1.
		Test 1 calculates E(A)->A v.s. E(A)  A. The earlier one is preferred.
		For nodiag=False, because the function expects significant eQTLs, p1 always return 1.
		For nodiag=True, uses diagonal elements of p2.
		Consider replacing p1 with your own (1-FDR) from eQTL discovery.
	p2:	numpy.ndarray((nt,nt2),dtype=ftype(='=f4' by default)). Probability for test 2.
		Test 2 calculates E(A)->A--B with E(A)->B v.s. E(A)->A<-B. The earlier one is preferred.
	p3:	numpy.ndarray((nt,nt2),dtype=ftype(='=f4' by default)). Probability for test 3.
		Test 3 calculates E(A)->A--B with E(A)->B v.s. E(A)->A->B. The latter one is preferred.
	p4:	numpy.ndarray((nt,nt2),dtype=ftype(='=f4' by default)). Probability for test 4.
		Test 4 calculates E(A)->A--B with E(A)->B v.s. E(A)->A  B. The earlier one is preferred.
	p5:	numpy.ndarray((nt,nt2),dtype=ftype(='=f4' by default)). Probability for test 5.
		Test 5 calculates E(A)->A--B with E(A)->B v.s. B<-E(A)->A. The earlier one is preferred.
	For more information on tests, see paper.
	ftype and gtype can be found in auto.py.
	
	Example: see findr.examples.geuvadis4
	"""
	from exceptions import ValueError
	if self.lib is None:
		raise ValueError("Not initialized.")
	import numpy as np
	from .auto import ftype_np,gtype_np
	if dg.dtype.char!=gtype_np:
		raise ValueError('Wrong input dtype for genotype data: dg.dtype.char is '+dg.dtype.char+'!='+gtype_np)
	if dt.dtype.char!=ftype_np or dt2.dtype.char!=ftype_np:
		raise ValueError('Wrong input dtype for gene expression data')
	if len(dg.shape)!=2 or len(dt.shape)!=2 or len(dt2.shape)!=2:
		raise ValueError('Wrong input shape')
	if type(nodiag) is not bool:
		raise ValueError('Wrong nodiag type')
	if not (type(memlimit) is int or type(memlimit) is long):
		raise ValueError('Wrong memlimit type')
	if not (na is None or type(na) is int or type(na) is long):
		raise ValueError('Wrong na type')
	if na is not None and na<=0:
		raise ValueError('Input requires na>0.')
	ng=dg.shape[0]
	nt=dt2.shape[0]
	ns=dg.shape[1]
	nvx=na+1 if na else dg.max()+1
	nd=1 if nodiag else 0
	
	if nvx<2:
		raise ValueError('Invalid genotype values')
	if dt.shape!=dg.shape or dt2.shape[1]!=ns:
		raise ValueError('Wrong input shape')
	if np.isnan(dt).sum()+np.isnan(dt2).sum()>0:
		raise ValueError('NaN found.')
	
	arglist=['const MATRIXG*','const MATRIXF*','const MATRIXF*','VECTORF*','MATRIXF*','MATRIXF*','MATRIXF*','MATRIXF*','size_t','byte','size_t']
	dgr=np.require(dg,requirements=['A','C','O','W'])
	dtr=np.require(dt,requirements=['A','C','O','W'])
	dt2r=np.require(dt2,requirements=['A','C','O','W'])
	d1=np.require(np.zeros(ng,dtype=dt.dtype),requirements=['A','C','O','W'])
	d2=np.require(np.zeros((ng,nt),dtype=dt.dtype),requirements=['A','C','O','W'])
	d3=np.require(np.zeros((ng,nt),dtype=dt.dtype),requirements=['A','C','O','W'])
	d4=np.require(np.zeros((ng,nt),dtype=dt.dtype),requirements=['A','C','O','W'])
	d5=np.require(np.zeros((ng,nt),dtype=dt.dtype),requirements=['A','C','O','W'])
	args=[dgr,dtr,dt2r,d1,d2,d3,d4,d5,nvx,nd,memlimit]
	func=self.cfunc("pijs_gassist",rettype='int',argtypes=arglist)
	ret=func(*args)
	ans={'ret':ret,'p1':d1,'p2':d2,'p3':d3,'p4':d4,'p5':d5}
	return ans

def _gassist_any(self,dg,dt,dt2,name,na=None,nodiag=False,memlimit=-1):
	"""Calculates probability of gene i regulating gene j with genotype data assisted method,
	with the recommended combination of multiple tests.
	dg:	numpy.ndarray(nt,ns,dtype=gtype(='u1' by default)) Genotype data.
		Entry dg[i,j] is genotype i's value for sample j.
		Each value must be among 0,1,...,na.
		Genotype i must be best (and significant) eQTL of gene i (in dt).
	dt:	numpy.ndarray(nt,ns,dtype=ftype(='=f4' by default)) Gene expression data for A
		Entry dt[i,j] is gene i's expression level for sample j.
		Genotype i (in dg) must be best (and significant) eQTL of gene i.
	dt2:numpy.ndarray(nt2,ns,dtype=ftype(='=f4' by default)) Gene expression data for B.
		dt2 has the same format as dt, and can be identical with, different from, or a superset of dt.
		When dt2 is a superset of (or identical with) dt, dt2 must be arranged
		to be identical with dt at its upper submatrix, i.e. dt2[:nt,:]=dt, and
		set parameter nodiag = 1.
	name:	actual C function name to call 
	na:	Number of alleles the species have. It determintes the maximum number of values each genotype can take. When unspecified, it is automatically
		determined as the maximum of dg.
	nodiag:	skip diagonal regulations, i.e. regulation A->B for A=B.
		This should be set to True when A is a subset of B and aligned correspondingly.
	memlimit:	The approximate memory usage limit in bytes for the library.  For datasets require a larger memory, calculation will be split into smaller chunks. If the memory limit is smaller than minimum required, calculation can fail with an error message. memlimit=0 defaults to unlimited memory usage.
	Return:	dictionary with following keys:
	ret:0 iff execution succeeded.
	p:	numpy.ndarray((nt,nt2),dtype=ftype(='=f4' by default)).
		Probability function from for recommended combination of multiple tests.
	For more information on tests, see paper.
	ftype and gtype can be found in auto.py.
	"""
	from exceptions import ValueError
	if self.lib is None:
		raise ValueError("Not initialized.")
	import numpy as np
	from .auto import ftype_np,gtype_np
	if dg.dtype.char!=gtype_np:
		raise ValueError('Wrong input dtype for genotype data: dg.dtype.char is '+dg.dtype.char+'!='+gtype_np)
	if dt.dtype.char!=ftype_np or dt2.dtype.char!=ftype_np:
		raise ValueError('Wrong input dtype for gene expression data')
	if len(dg.shape)!=2 or len(dt.shape)!=2 or len(dt2.shape)!=2:
		raise ValueError('Wrong input shape')
	if type(nodiag) is not bool:
		raise ValueError('Wrong nodiag type')
	if not (type(memlimit) is int or type(memlimit) is long):
		raise ValueError('Wrong memlimit type')
	if not (na is None or type(na) is int or type(na) is long):
		raise ValueError('Wrong na type')
	if na is not None and na<=0:
		raise ValueError('Input requires na>0.')
	ng=dg.shape[0]
	nt=dt2.shape[0]
	ns=dg.shape[1]
	nvx=na+1 if na else dg.max()+1
	nd=1 if nodiag else 0
	
	if nvx<2:
		raise ValueError('Invalid genotype values')
	if dt.shape!=dg.shape or dt2.shape[1]!=ns:
		raise ValueError('Wrong input shape')
	if np.isnan(dt).sum()+np.isnan(dt2).sum()>0:
		raise ValueError('NaN found.')

	func=self.cfunc(name,rettype='int',argtypes=['const MATRIXG*','const MATRIXF*','const MATRIXF*','MATRIXF*','size_t','byte','size_t'])
	d=np.require(np.zeros((ng,nt),dtype=dt.dtype),requirements=['A','C','O','W'])
	dgr=np.require(dg,requirements=['A','C','O','W'])
	dtr=np.require(dt,requirements=['A','C','O','W'])
	dt2r=np.require(dt2,requirements=['A','C','O','W'])
	ret=func(dgr,dtr,dt2r,d,nvx,nd,memlimit)
	ans={'ret':ret,'p':d}
	return ans

def gassist(self,dg,dt,dt2,na=None,nodiag=False,memlimit=-1):
	"""Calculates probability of gene i regulating gene j with genotype data assisted method,
	with the recommended combination of multiple tests.
	Probabilities are converted from likelihood ratios separately for each A. This gives better
	predictions when the number of secondary targets (dt2) is large. (Check program warnings.)
	dg:	numpy.ndarray(nt,ns,dtype=gtype(='u1' by default)) Genotype data.
		Entry dg[i,j] is genotype i's value for sample j.
		Each value must be among 0,1,...,na.
		Genotype i must be best (and significant) eQTL of gene i (in dt).
	dt:	numpy.ndarray(nt,ns,dtype=ftype(='=f4' by default)) Gene expression data for A
		Entry dt[i,j] is gene i's expression level for sample j.
		Genotype i (in dg) must be best (and significant) eQTL of gene i.
	dt2:numpy.ndarray(nt2,ns,dtype=ftype(='=f4' by default)) Gene expression data for B.
		dt2 has the same format as dt, and can be identical with, different from, or a superset of dt.
		When dt2 is a superset of (or identical with) dt, dt2 must be arranged
		to be identical with dt at its upper submatrix, i.e. dt2[:nt,:]=dt, and
		set parameter nodiag = 1.
	na:	Number of alleles the species have. It determintes the maximum number of values each genotype can take. When unspecified, it is automatically
		determined as the maximum of dg.
	nodiag:	skip diagonal regulations, i.e. regulation A->B for A=B.
		This should be set to True when A is a subset of B and aligned correspondingly.
	memlimit:	The approximate memory usage limit in bytes for the library.  For datasets require a larger memory, calculation will be split into smaller chunks. If the memory limit is smaller than minimum required, calculation can fail with an error message. memlimit=0 defaults to unlimited memory usage.
	Return:	dictionary with following keys:
	ret:0 iff execution succeeded.
	p:	numpy.ndarray((nt,nt2),dtype=ftype(='=f4' by default)).
		Probability function from for recommended combination of multiple tests.
	For more information on tests, see paper.
	ftype and gtype can be found in auto.py.
	
	Example: see findr.examples.geuvadis2, findr.examples.geuvadis3
	"""
	return _gassist_any(self,dg,dt,dt2,"pij_gassist",na=na,nodiag=nodiag,memlimit=memlimit)

def gassist_trad(self,dg,dt,dt2,na=None,nodiag=False,memlimit=-1):
	"""Calculates probability of gene i regulating gene j with genotype data assisted method with traditional test.
	WARNING: This is not and is not intended as a loyal reimplementation of Trigger. This test does not include p1.
	dg:	numpy.ndarray(nt,ns,dtype=gtype(='u1' by default)) Genotype data.
		Entry dg[i,j] is genotype i's value for sample j.
		Each value must be among 0,1,...,na.
		Genotype i must be best (and significant) eQTL of gene i (in dt).
	dt:	numpy.ndarray(nt,ns,dtype=ftype(='=f4' by default)) Gene expression data for A
		Entry dt[i,j] is gene i's expression level for sample j.
		Genotype i (in dg) must be best (and significant) eQTL of gene i.
	dt2:numpy.ndarray(nt2,ns,dtype=ftype(='=f4' by default)) Gene expression data for B.
		dt2 has the same format as dt, and can be identical with, different from, or a superset of dt.
		When dt2 is a superset of (or identical with) dt, dt2 must be arranged
		to be identical with dt at its upper submatrix, i.e. dt2[:nt,:]=dt, and
		set parameter nodiag = 1.
	na:	Number of alleles the species have. It determintes the maximum number of values each genotype can take. When unspecified, it is automatically
		determined as the maximum of dg.
	nodiag:	skip diagonal regulations, i.e. regulation A->B for A=B.
		This should be set to True when A is a subset of B and aligned correspondingly.
	memlimit:	The approximate memory usage limit in bytes for the library.  For datasets require a larger memory, calculation will be split into smaller chunks. If the memory limit is smaller than minimum required, calculation can fail with an error message. memlimit=0 defaults to unlimited memory usage.
	Return:	dictionary with following keys:
	ret:0 iff execution succeeded.
	p:	numpy.ndarray((nt,nt2),dtype=ftype(='=f4' by default)).
		Probability function from for recommended combination of multiple tests.
	For more information on tests, see paper.
	ftype and gtype can be found in auto.py.
	
	Example: see findr.examples.geuvadis2, findr.examples.geuvadis3 (same format)
	"""
	return _gassist_any(self,dg,dt,dt2,"pij_gassist_trad",na=na,nodiag=nodiag,memlimit=memlimit)

def cassists_pv(self,dc,dt,dt2,memlimit=-1):
	"""Calculates p-values of gene i regulating gene j with continuous anchor data assisted method with multiple tests.
	dc:	numpy.ndarray(nt,ns,dtype=ftype(='f4' by default)) Continuous anchor data.
		Entry dc[i,j] is anchor i's value for sample j.
		Anchor i is used to infer the probability of gene i -> any other gene.
	dt:	numpy.ndarray(nt,ns,dtype=ftype(='=f4' by default)) Gene expression data for A
		Entry dt[i,j] is gene i's expression level for sample j.
	dt2:numpy.ndarray(nt2,ns,dtype=ftype(='=f4' by default)) Gene expression data for B.
		dt2 has the same format as dt, and can be identical with, different from, or a superset of dt.
	memlimit:	The approximate memory usage limit in bytes for the library. If the memory limit is smaller than minimum required, calculation can fail with an error message. memlimit=0 defaults to unlimited memory usage.
	Return:	dictionary with following keys:
	ret:0 iff execution succeeded.
	p1:	numpy.ndarray(nt,dtype=ftype(='f4' by default)). P-values for LLR of test 1.
		Test 1 calculates E(A)->A v.s. E(A)  A.
	p2:	numpy.ndarray((nt,nt2),dtype=ftype(='f4' by default)). P-values for LLR of test 2.
		Test 2 calculates E(A)->A--B with E(A)->B v.s. E(A)->A<-B.
	p3:	numpy.ndarray((nt,nt2),dtype=ftype(='f4' by default)). P-values for LLR of test 3.
		Test 3 calculates E(A)->A--B with E(A)->B v.s. E(A)->A->B.
	p4:	numpy.ndarray((nt,nt2),dtype=ftype(='f4' by default)). P-values for LLR of test 4.
		Test 4 calculates E(A)->A--B with E(A)->B v.s. E(A)->A  B.
	p5:	numpy.ndarray((nt,nt2),dtype=ftype(='f4' by default)). P-values for LLR of test 5.
		Test 5 calculates E(A)->A--B with E(A)->B v.s. B<-E(A)->A.
	For more information on tests, see paper.
	ftype can be found in auto.py.
	
	Example: see findr.examples.geuvadis6 (similar format)
	"""
	from exceptions import ValueError
	if self.lib is None:
		raise ValueError("Not initialized.")
	import numpy as np
	from .auto import ftype_np
	if dc.dtype.char!=ftype_np or dt.dtype.char!=ftype_np or dt2.dtype.char!=ftype_np:
		raise ValueError('Wrong input dtype for gene expression data')
	if len(dc.shape)!=2 or len(dt.shape)!=2 or len(dt2.shape)!=2:
		raise ValueError('Wrong input shape')
	if not (type(memlimit) is int or type(memlimit) is long):
		raise ValueError('Wrong memlimit type')
	ng=dc.shape[0]
	nt=dt2.shape[0]
	ns=dc.shape[1]
	
	if dt.shape!=dc.shape or dt2.shape[1]!=ns:
		raise ValueError('Wrong input shape')
	if np.isnan(dc).sum()+np.isnan(dt).sum()+np.isnan(dt2).sum()>0:
		raise ValueError('NaN found.')
	
	arglist=['const MATRIXF*','const MATRIXF*','const MATRIXF*','VECTORF*','MATRIXF*','MATRIXF*','MATRIXF*','MATRIXF*','size_t']

	dcr=np.require(dc,requirements=['A','C','O','W'])
	dtr=np.require(dt,requirements=['A','C','O','W'])
	dt2r=np.require(dt2,requirements=['A','C','O','W'])
	d1=np.require(np.zeros(ng,dtype=dt.dtype),requirements=['A','C','O','W'])
	d2=np.require(np.zeros((ng,nt),dtype=dt.dtype),requirements=['A','C','O','W'])
	d3=np.require(np.zeros((ng,nt),dtype=dt.dtype),requirements=['A','C','O','W'])
	d4=np.require(np.zeros((ng,nt),dtype=dt.dtype),requirements=['A','C','O','W'])
	d5=np.require(np.zeros((ng,nt),dtype=dt.dtype),requirements=['A','C','O','W'])
	args=[dcr,dtr,dt2r,d1,d2,d3,d4,d5,memlimit]
	func=self.cfunc('pijs_cassist_pv',rettype='int',argtypes=arglist)
	ret=func(*args)
	ans={'ret':ret,'p1':d1,'p2':d2,'p3':d3,'p4':d4,'p5':d5}
	return ans
	
def _cassists_any(self,dc,dt,dt2,name,nodiag=False,memlimit=-1):
	"""Calculates probability of gene i regulating gene j with continuous anchor data assisted method,
	with multiple tests, by converting log likelihoods into probabilities per A for all B.
	dc:	numpy.ndarray(nt,ns,dtype=ftype(='f4' by default)) Continuous anchor data.
		Entry dc[i,j] is anchor i's value for sample j.
		Anchor i is used to infer the probability of gene i -> any other gene.
	dt:	numpy.ndarray(nt,ns,dtype=ftype(='=f4' by default)) Gene expression data for A
		Entry dt[i,j] is gene i's expression level for sample j.
	dt2:numpy.ndarray(nt2,ns,dtype=ftype(='=f4' by default)) Gene expression data for B.
		dt2 has the same format as dt, and can be identical with, different from, or a superset of dt.
		When dt2 is a superset of (or identical with) dt, dt2 must be arranged
		to be identical with dt at its upper submatrix, i.e. dt2[:nt,:]=dt, and
		set parameter nodiag = 1.
	name:	actual C function name to call 
	nodiag:	skip diagonal regulations, i.e. regulation A->B for A=B.
		This should be set to True when A is a subset of B and aligned correspondingly.
	memlimit:	The approximate memory usage limit in bytes for the library.  For datasets require a larger memory, calculation will be split into smaller chunks. If the memory limit is smaller than minimum required, calculation can fail with an error message. memlimit=0 defaults to unlimited memory usage.
	Return:	dictionary with following keys:
	ret:0 iff execution succeeded.
	p1:	numpy.ndarray(nt,dtype=ftype(='=f4' by default)). Probability for test 1.
		Test 1 calculates E(A)->A v.s. E(A)  A. The earlier one is preferred.
		For nodiag=False, because the function expects significant anchors, p1 always return 1.
		For nodiag=True, uses diagonal elements of p2.
	p2:	numpy.ndarray((nt,nt2),dtype=ftype(='=f4' by default)). Probability for test 2.
		Test 2 calculates E(A)->A--B with E(A)->B v.s. E(A)->A<-B. The earlier one is preferred.
	p3:	numpy.ndarray((nt,nt2),dtype=ftype(='=f4' by default)). Probability for test 3.
		Test 3 calculates E(A)->A--B with E(A)->B v.s. E(A)->A->B. The latter one is preferred.
	p4:	numpy.ndarray((nt,nt2),dtype=ftype(='=f4' by default)). Probability for test 4.
		Test 4 calculates E(A)->A--B with E(A)->B v.s. E(A)->A  B. The earlier one is preferred.
	p5:	numpy.ndarray((nt,nt2),dtype=ftype(='=f4' by default)). Probability for test 5.
		Test 5 calculates E(A)->A--B with E(A)->B v.s. B<-E(A)->A. The earlier one is preferred.
	For more information on tests, see paper.
	ftype can be found in auto.py.
	"""
	from exceptions import ValueError
	if self.lib is None:
		raise ValueError("Not initialized.")
	import numpy as np
	from .auto import ftype_np
	if dc.dtype.char!=ftype_np or dt.dtype.char!=ftype_np or dt2.dtype.char!=ftype_np:
		raise ValueError('Wrong input dtype for gene expression data')
	if len(dc.shape)!=2 or len(dt.shape)!=2 or len(dt2.shape)!=2:
		raise ValueError('Wrong input shape')
	if type(nodiag) is not bool:
		raise ValueError('Wrong nodiag type')
	if not (type(memlimit) is int or type(memlimit) is long):
		raise ValueError('Wrong memlimit type')
	ng=dc.shape[0]
	nt=dt2.shape[0]
	ns=dc.shape[1]
	nd=1 if nodiag else 0
	
	if dt.shape!=dc.shape or dt2.shape[1]!=ns:
		raise ValueError('Wrong input shape')
	if np.isnan(dc).sum()+np.isnan(dt).sum()+np.isnan(dt2).sum()>0:
		raise ValueError('NaN found.')
	
	arglist=['const MATRIXF*','const MATRIXF*','const MATRIXF*','VECTORF*','MATRIXF*','MATRIXF*','MATRIXF*','MATRIXF*','byte','size_t']
	names=name
	dcr=np.require(dc,requirements=['A','C','O','W'])
	dtr=np.require(dt,requirements=['A','C','O','W'])
	dt2r=np.require(dt2,requirements=['A','C','O','W'])
	d1=np.require(np.zeros(ng,dtype=dt.dtype),requirements=['A','C','O','W'])
	d2=np.require(np.zeros((ng,nt),dtype=dt.dtype),requirements=['A','C','O','W'])
	d3=np.require(np.zeros((ng,nt),dtype=dt.dtype),requirements=['A','C','O','W'])
	d4=np.require(np.zeros((ng,nt),dtype=dt.dtype),requirements=['A','C','O','W'])
	d5=np.require(np.zeros((ng,nt),dtype=dt.dtype),requirements=['A','C','O','W'])
	args=[dcr,dtr,dt2r,d1,d2,d3,d4,d5,nd,memlimit]
	func=self.cfunc(names,rettype='int',argtypes=arglist)
	ret=func(*args)
	ans={'ret':ret,'p1':d1,'p2':d2,'p3':d3,'p4':d4,'p5':d5}
	return ans

def cassists(self,dc,dt,dt2,nodiag=False,memlimit=-1):
	"""Calculates probability of gene i regulating gene j with continuous data assisted method,
	with multiple tests, by converting log likelihoods into probabilities per A for all B.
	Probabilities are converted from likelihood ratios separately for each A. This gives better
	predictions when the number of secondary targets (dt2) is large. (Check program warnings.)
	dc:	numpy.ndarray(nt,ns,dtype=ftype(='f4' by default)) Continuous anchor data.
		Entry dc[i,j] is anchor i's value for sample j.
		Anchor i is used to infer the probability of gene i -> any other gene.
	dt:	numpy.ndarray(nt,ns,dtype=ftype(='=f4' by default)) Gene expression data for A
		Entry dt[i,j] is gene i's expression level for sample j.
	dt2:numpy.ndarray(nt2,ns,dtype=ftype(='=f4' by default)) Gene expression data for B.
		dt2 has the same format as dt, and can be identical with, different from, or a superset of dt.
		When dt2 is a superset of (or identical with) dt, dt2 must be arranged
		to be identical with dt at its upper submatrix, i.e. dt2[:nt,:]=dt, and
		set parameter nodiag = 1.
	nodiag:	skip diagonal regulations, i.e. regulation A->B for A=B.
		This should be set to True when A is a subset of B and aligned correspondingly.
	memlimit:	The approximate memory usage limit in bytes for the library.  For datasets require a larger memory, calculation will be split into smaller chunks. If the memory limit is smaller than minimum required, calculation can fail with an error message. memlimit=0 defaults to unlimited memory usage.
	Return:	dictionary with following keys:
	ret:0 iff execution succeeded.
	p1:	numpy.ndarray(nt,dtype=ftype(='=f4' by default)). Probability for test 1.
		Test 1 calculates E(A)->A v.s. E(A)  A. The earlier one is preferred.
		For nodiag=False, because the function expects significant anchors, p1 always return 1.
		For nodiag=True, uses diagonal elements of p2.
	p2:	numpy.ndarray((nt,nt2),dtype=ftype(='=f4' by default)). Probability for test 2.
		Test 2 calculates E(A)->A--B with E(A)->B v.s. E(A)->A<-B. The earlier one is preferred.
	p3:	numpy.ndarray((nt,nt2),dtype=ftype(='=f4' by default)). Probability for test 3.
		Test 3 calculates E(A)->A--B with E(A)->B v.s. E(A)->A->B. The latter one is preferred.
	p4:	numpy.ndarray((nt,nt2),dtype=ftype(='=f4' by default)). Probability for test 4.
		Test 4 calculates E(A)->A--B with E(A)->B v.s. E(A)->A  B. The earlier one is preferred.
	p5:	numpy.ndarray((nt,nt2),dtype=ftype(='=f4' by default)). Probability for test 5.
		Test 5 calculates E(A)->A--B with E(A)->B v.s. B<-E(A)->A. The earlier one is preferred.
	For more information on tests, see paper.
	ftype can be found in auto.py.
	
	Example: see findr.examples.geuvadis4 (similar format)
	"""
	return _cassists_any(self,dc,dt,dt2,"pijs_cassist",nodiag=nodiag,memlimit=memlimit)

def _cassist_any(self,dc,dt,dt2,name,nodiag=False,memlimit=-1):
	"""Calculates probability of gene i regulating gene j with continuous data assisted method,
	with the recommended combination of multiple tests.
	dc:	numpy.ndarray(nt,ns,dtype=ftype(='f4' by default)) Continuous anchor data.
		Entry dc[i,j] is anchor i's value for sample j.
		Anchor i is used to infer the probability of gene i -> any other gene.
	dt:	numpy.ndarray(nt,ns,dtype=ftype(='=f4' by default)) Gene expression data for A
		Entry dt[i,j] is gene i's expression level for sample j.
	dt2:numpy.ndarray(nt2,ns,dtype=ftype(='=f4' by default)) Gene expression data for B.
		dt2 has the same format as dt, and can be identical with, different from, or a superset of dt.
		When dt2 is a superset of (or identical with) dt, dt2 must be arranged
		to be identical with dt at its upper submatrix, i.e. dt2[:nt,:]=dt, and
		set parameter nodiag = 1.
	name:	actual C function name to call 
	nodiag:	skip diagonal regulations, i.e. regulation A->B for A=B.
		This should be set to True when A is a subset of B and aligned correspondingly.
	memlimit:	The approximate memory usage limit in bytes for the library.  For datasets require a larger memory, calculation will be split into smaller chunks. If the memory limit is smaller than minimum required, calculation can fail with an error message. memlimit=0 defaults to unlimited memory usage.
	Return:	dictionary with following keys:
	ret:0 iff execution succeeded.
	p:	numpy.ndarray((nt,nt2),dtype=ftype(='=f4' by default)).
		Probability function from for recommended combination of multiple tests.
	For more information on tests, see paper.
	ftype can be found in auto.py.
	"""
	from exceptions import ValueError
	if self.lib is None:
		raise ValueError("Not initialized.")
	import numpy as np
	from .auto import ftype_np
	if dc.dtype.char!=ftype_np or dt.dtype.char!=ftype_np or dt2.dtype.char!=ftype_np:
		raise ValueError('Wrong input dtype for gene expression data')
	if len(dc.shape)!=2 or len(dt.shape)!=2 or len(dt2.shape)!=2:
		raise ValueError('Wrong input shape')
	if type(nodiag) is not bool:
		raise ValueError('Wrong nodiag type')
	if not (type(memlimit) is int or type(memlimit) is long):
		raise ValueError('Wrong memlimit type')
	ng=dc.shape[0]
	nt=dt2.shape[0]
	ns=dc.shape[1]
	nd=1 if nodiag else 0
	
	if dt.shape!=dc.shape or dt2.shape[1]!=ns:
		raise ValueError('Wrong input shape')
	if np.isnan(dc).sum()+np.isnan(dt).sum()+np.isnan(dt2).sum()>0:
		raise ValueError('NaN found.')

	func=self.cfunc(name,rettype='int',argtypes=['const MATRIXF*','const MATRIXF*','const MATRIXF*','MATRIXF*','byte','size_t'])
	d=np.require(np.zeros((ng,nt),dtype=dt.dtype),requirements=['A','C','O','W'])
	dcr=np.require(dc,requirements=['A','C','O','W'])
	dtr=np.require(dt,requirements=['A','C','O','W'])
	dt2r=np.require(dt2,requirements=['A','C','O','W'])
	ret=func(dcr,dtr,dt2r,d,nd,memlimit)
	ans={'ret':ret,'p':d}
	return ans

def cassist(self,dc,dt,dt2,nodiag=False,memlimit=-1):
	"""Calculates probability of gene i regulating gene j with continuous data assisted method,
	with the recommended combination of multiple tests.
	Probabilities are converted from likelihood ratios separately for each A. This gives better
	predictions when the number of secondary targets (dt2) is large. (Check program warnings.)
	dc:	numpy.ndarray(nt,ns,dtype=ftype(='f4' by default)) Continuous anchor data.
		Entry dc[i,j] is anchor i's value for sample j.
		Anchor i is used to infer the probability of gene i -> any other gene.
	dt:	numpy.ndarray(nt,ns,dtype=ftype(='=f4' by default)) Gene expression data for A
		Entry dt[i,j] is gene i's expression level for sample j.
	dt2:numpy.ndarray(nt2,ns,dtype=ftype(='=f4' by default)) Gene expression data for B.
		dt2 has the same format as dt, and can be identical with, different from, or a superset of dt.
		When dt2 is a superset of (or identical with) dt, dt2 must be arranged
		to be identical with dt at its upper submatrix, i.e. dt2[:nt,:]=dt, and
		set parameter nodiag = 1.
	nodiag:	skip diagonal regulations, i.e. regulation A->B for A=B.
		This should be set to True when A is a subset of B and aligned correspondingly.
	memlimit:	The approximate memory usage limit in bytes for the library.  For datasets require a larger memory, calculation will be split into smaller chunks. If the memory limit is smaller than minimum required, calculation can fail with an error message. memlimit=0 defaults to unlimited memory usage.
	Return:	dictionary with following keys:
	ret:0 iff execution succeeded.
	p:	numpy.ndarray((nt,nt2),dtype=ftype(='=f4' by default)).
		Probability function from for recommended combination of multiple tests.
	For more information on tests, see paper.
	ftype can be found in auto.py.
	
	Example: see findr.examples.geuvadis5
	"""
	return _cassist_any(self,dc,dt,dt2,"pij_cassist",nodiag=nodiag,memlimit=memlimit)

def cassist_trad(self,dc,dt,dt2,nodiag=False,memlimit=-1):
	"""Calculates probability of gene i regulating gene j with continuous data assisted method with traditional test.
	dc:	numpy.ndarray(nt,ns,dtype=ftype(='f4' by default)) Continuous anchor data.
		Entry dc[i,j] is anchor i's value for sample j.
		Anchor i is used to infer the probability of gene i -> any other gene.
	dt:	numpy.ndarray(nt,ns,dtype=ftype(='=f4' by default)) Gene expression data for A
		Entry dt[i,j] is gene i's expression level for sample j.
	dt2:numpy.ndarray(nt2,ns,dtype=ftype(='=f4' by default)) Gene expression data for B.
		dt2 has the same format as dt, and can be identical with, different from, or a superset of dt.
		When dt2 is a superset of (or identical with) dt, dt2 must be arranged
		to be identical with dt at its upper submatrix, i.e. dt2[:nt,:]=dt, and
		set parameter nodiag = 1.
	nodiag:	skip diagonal regulations, i.e. regulation A->B for A=B.
		This should be set to True when A is a subset of B and aligned correspondingly.
	memlimit:	The approximate memory usage limit in bytes for the library.  For datasets require a larger memory, calculation will be split into smaller chunks. If the memory limit is smaller than minimum required, calculation can fail with an error message. memlimit=0 defaults to unlimited memory usage.
	Return:	dictionary with following keys:
	ret:0 iff execution succeeded.
	p:	numpy.ndarray((nt,nt2),dtype=ftype(='=f4' by default)).
		Probability function from for recommended combination of multiple tests.
	For more information on tests, see paper.
	ftype can be found in auto.py.
	
	Example: see findr.examples.geuvadis5 (same format)
	"""
	return _cassist_any(self,dc,dt,dt2,"pij_cassist_trad",nodiag=nodiag,memlimit=memlimit)

def rank_pv(self,dt,dt2,memlimit=-1):
	"""Calculates p-values of gene i correlating with gene j by converting log likelihoods into probabilities per A for all B.
	dt:	numpy.ndarray(nt,ns,dtype=ftype(='=f4' by default)) Gene expression data for A
		Entry dt[i,j] is gene i's expression level for sample j.
	dt2:numpy.ndarray(nt2,ns,dtype=ftype(='=f4' by default)) Gene expression data for B.
		dt2 has the same format as dt, and can be identical with, different from, a subset of, or a superset of dt.
	memlimit:	The approximate memory usage limit in bytes for the library.  For datasets require a larger memory, calculation will fail with an error message. memlimit=0 defaults to unlimited memory usage.
	Return:	dictionary with following keys:
	ret:0 iff execution succeeded.
	p:	numpy.ndarray((nt,nt2),dtype=ftype(='=f4' by default)). P-values for A--B.
	ftype and gtype can be found in auto.py.
	
	Example: see findr.examples.geuvadis1 (similar format)
	"""
	from exceptions import ValueError
	if self.lib is None:
		raise ValueError("Not initialized.")
	import numpy as np
	from .auto import ftype_np,gtype_np
	if dt.dtype.char!=ftype_np or dt2.dtype.char!=ftype_np:
		raise ValueError('Wrong input dtype for gene expression data')
	if len(dt.shape)!=2 or len(dt2.shape)!=2:
		raise ValueError('Wrong input shape')
	if not (type(memlimit) is int or type(memlimit) is long):
		raise ValueError('Wrong memlimit type')
	ng=dt.shape[0]
	nt=dt2.shape[0]
	ns=dt.shape[1]
	
	if dt2.shape[1]!=ns:
		raise ValueError('Wrong input shape')
	if np.isnan(dt).sum()+np.isnan(dt2).sum()>0:
		raise ValueError('NaN found.')

	dp=np.require(np.zeros((ng,nt),dtype=dt.dtype),requirements=['A','C','O','W'])
	dtr=np.require(dt,requirements=['A','C','O','W'])
	dt2r=np.require(dt2,requirements=['A','C','O','W'])
	arglist=['const MATRIXF*','const MATRIXF*','MATRIXF*','size_t']
	args=[dtr,dt2r,dp,memlimit]
	func=self.cfunc('pij_rank_pv',rettype='int',argtypes=arglist)
	ret=func(*args)
	ans={'ret':ret,'p':dp}
	return ans

def rank(self,dt,dt2,nodiag=False,memlimit=-1):
	"""Calculates probability of gene i correlating with gene j by converting log likelihoods into probabilities per A for all B.
	dt:	numpy.ndarray(nt,ns,dtype=ftype(='=f4' by default)) Gene expression data for A
		Entry dt[i,j] is gene i's expression level for sample j.
	dt2:numpy.ndarray(nt2,ns,dtype=ftype(='=f4' by default)) Gene expression data for B.
		dt2 has the same format as dt, and can be identical with, different from, a subset of, or a superset of dt. When dt2 is a superset of (or identical with) dt, dt2 must be arranged to be identical with dt at its upper submatrix, i.e. dt2[:nt,:]=dt, and set parameter nodiag = 1. Similarly if dt2 is a subset of dt.
	nodiag:	skip diagonal regulations, i.e. regulation A--B for A=B.
		This should be set to True when A is a subset of B and aligned correspondingly.
	memlimit:	The approximate memory usage limit in bytes for the library.  For datasets require a larger memory, calculation will fail with an error message. memlimit=0 defaults to unlimited memory usage.
	Return:	dictionary with following keys:
	ret:0 iff execution succeeded.
	p:	numpy.ndarray((nt,nt2),dtype=ftype(='=f4' by default)). Probability for A--B.
	ftype and gtype can be found in auto.py.
	
	Example: see findr.examples.geuvadis1
	"""
	from exceptions import ValueError
	if self.lib is None:
		raise ValueError("Not initialized.")
	import numpy as np
	from .auto import ftype_np,gtype_np
	if dt.dtype.char!=ftype_np or dt2.dtype.char!=ftype_np:
		raise ValueError('Wrong input dtype for gene expression data')
	if len(dt.shape)!=2 or len(dt2.shape)!=2:
		raise ValueError('Wrong input shape')
	if type(nodiag) is not bool:
		raise ValueError('Wrong nodiag type')
	if not (type(memlimit) is int or type(memlimit) is long):
		raise ValueError('Wrong memlimit type')
	ng=dt.shape[0]
	nt=dt2.shape[0]
	ns=dt.shape[1]
	nd=1 if nodiag else 0
	
	if dt2.shape[1]!=ns:
		raise ValueError('Wrong input shape')
	if np.isnan(dt).sum()+np.isnan(dt2).sum()>0:
		raise ValueError('NaN found.')

	dp=np.require(np.zeros((ng,nt),dtype=dt.dtype),requirements=['A','C','O','W'])
	dtr=np.require(dt,requirements=['A','C','O','W'])
	dt2r=np.require(dt2,requirements=['A','C','O','W'])
	arglist=['const MATRIXF*','const MATRIXF*','MATRIXF*','byte','size_t']
	args=[dtr,dt2r,dp,nd,memlimit]
	func=self.cfunc('pij_rank',rettype='int',argtypes=arglist)
	ret=func(*args)
	ans={'ret':ret,'p':dp}
	return ans

































